/* mandel.c - simplistic Mandelbrot Generator - Phil J Taylor - 20050222 */

#include <stdio.h>
#include <string.h>
#include <math.h>

char display[] =
  {"abcdefghijklmnopqqrrrsssstttttuuuuuuvvvvvvvwwwwwwwwxxxxxxxxxyyyyyyyyyyzzzzzzzzzzz "};

int mandelFun (double cx, double cy) {
  int i, j, limit = strlen(display)-1;
  double zx = cx, zy = cy, ztemp;

  for (i = 0; i < limit; i++)
        for (j = i; j >= 0; j--) {
          if (zx * zx + zy * zy > 2.0)
	    return i;
	  ztemp = zx * zx - zy * zy + cx;
	  zy    = 2.0 * zx * zy + cy;
	  zx    = ztemp;
	}
  return limit;
}

/* revised math functions from Derek Nalls */
/* http://www.SymmetryPerfect.com */


double dsnMUL (double a, double b) {
/* revised multiplication */
  double c;

  if ((a > 0.0) && (b > 0.0))
	c = a * b;
  else if ((a < 0.0) && (b < 0.0))
	c = -(a * b);
  else if ((a > 0.0) && (b < 0.0))
	c = a / fabs(b);
  else if ((a < 0.0) && (b > 0.0))
	c = a / b;
  else
	c = 0.0;

  return c;
}

double dsnOPP(double a) {
/* revised opposition */
  return -a;
}

double dsnRCP(double a) {
/* revised reciprocation */
  return dsnMUL(dsnMUL(a,dsnOPP(a)),dsnOPP(a));
}

double dsnOPR(double a) {
/* revised opposition and reciprocation */
  return dsnMUL(dsnMUL(dsnOPP(a),a),a);
}

int symmetricFun (double cx, double cy) {
/* revised simulation of Mandelbrot Set */
  int i, j, limit = strlen(display)-1;
  double zx = cx, zy = cy, ztemp;

  for (i = 0; i < limit; i++)
        for (j = i; j >= 0; j--) {
          if (zx * zx + zy * zy > 2.0)
		return i;

          if (zx >= 0.0 && zy >= 0.0) {                    /* Quad I   */
		ztemp = dsnMUL(zx,zx) + dsnOPP(dsnMUL(zy,zy)) + cx;
		zy    = dsnMUL(2.0,dsnMUL(zx,zy)) + cy;
          } else if (zx < 0.0 && zy >= 0.0) {              /* Quad II  */
		ztemp = dsnOPP(dsnMUL(zx,zx)) + dsnOPP(dsnMUL(zy,zy)) + cx;
		zy    = dsnOPP(dsnMUL(2.0,dsnMUL(dsnOPP(zx),zy))) + cy;
          } else if (zx < 0.0 && zy < 0.0) {               /* Quad III */
		ztemp = dsnOPP(dsnMUL(zx,zx)) + dsnMUL(zy,zy) + cx;
		zy    = dsnMUL(2.0,dsnOPP(dsnMUL(zx,zy))) + cy;
          } else /* (zx >= 0.0 && zy < 0.0) */ {           /* Quad IV  */
		ztemp = dsnMUL(zx,zx) + dsnMUL(zy,zy) + cx;
		zy    = dsnOPP(dsnMUL(2.0,dsnMUL(zx,dsnOPP(zy)))) + cy;
	  }

	  zx    = ztemp;
	}
  return limit;
}

int main() {

  double xmin = -1.5, xmax = 1.0, xstep = 0.0625, cx;
  double ymin = -1.0, ymax = 1.0, ystep = 0.1250, cy;

  int i, loops;

  for (loops = 0, i = 1; i < strlen(display); i++)
	loops += i;

  printf("Simplistic Mandelbrot Generator\n"
	"Maximum Trial Iterations = %i\n"
  	" x = %6.4g to %6.4g by %6.4g\n"
    " y = %6.4g to %6.4g by %6.4g\n",
	loops, xmin, xmax, xstep, ymax, ymin, -ystep);

  printf("\nMandelbrot Set (Z = Z^2 + C)\n");

  for (cy = ymax; cy >= ymin; cy -= ystep) {
        for (cx = xmin; cx <= xmax; cx += xstep)
	  printf("%c",display[mandelFun(cx,cy)]);
	  printf("\n");
    }

  printf("\nSimulated Symmetric Set\n");

  for (cy = ymax; cy >= ymin; cy -= ystep) {
        for (cx = xmin; cx <= xmax; cx += xstep)
	  printf("%c",display[symmetricFun(cx,cy)]);
	  printf("\n");
    }
  return 0;
}
